#ifndef CDF_HEADER
#define CDF_HEADER

/* ==== HEADER cdf.h ==== */

/**
 * \file cdf.h
 * Routine collection for estimating cumulative distribution functions.
 */

/* ANSI C, IRIX 4.0.5, 1. Oct. 1993. Andras Aszodi */

/* ---- STANDARD HEADERS ---- */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <float.h>

/* ---- GLOBAL TYPES ---- */

#ifdef __cplusplus
namespace RazorBack {
#endif

/** 
 * Cdfrec_ : this structure holds the variables for estimating a CDF.
 */
typedef struct
{
    double *X;	/**< array of independent variable points */
    double *Y;	/**< array holding the estimated CDF */
    int *Yint;	/**< array for raw counts */
    int N, Bno; /**< total number of points seen, and number of bins */
}
Cdfrec_ ;

/* ---- PROTOTYPES ---- */

#ifdef __cplusplus
extern "C" {
#endif

/**
 * init_cdf(Binno, Low, Up, Cdf):
 * initialises Cdf to collect data between Low and Up
 * into Binno bins. X[] is initialised to
 * uniform sampling but the user can overwrite it after the call
 * to init_cdf(), the only rule being that if i<j then X[i]<=X[j].
 * If Low>Up, they are
 * silently swapped. In C++ this would be a constructor.
 * Return value: 0 if OK, -1 if some of the parameters were silly, 
 * -2 if no more memory could be allocated.
 */
int init_cdf(int Binno, double Low, double Up, Cdfrec_ *Cdf);

/**
 * add_cdf(Value, Cdf):
 * enters the value Value into Cdf. The bin which is
 * the first that is higher than Value is incremented by 1.
 * Return value: 0 if OK, 1 if it is too large.
 */
int add_cdf(double Value, Cdfrec_ *Cdf);

/**
 * eval_cdf(Cdf):
 * calculates the CDF and puts the estimated values 
 * in Cdf->Y[] (a normalisation). Leaves the original Cdf->Yint[]
 * intact.
 * Return value: 0 if OK, -1 if no data were in Cdf.
 */
int eval_cdf(Cdfrec_ *Cdf);

/**
 * remove_cdf(Cdf):
 * frees up the 3 arrays in Cdfrec_ and resets all
 * variables to 0. Does not remove Cdf itself!
 * This would be a C++ destructor.
 */
void remove_cdf(Cdfrec_ *Cdf);

#ifdef __cplusplus
} }
#endif

/* ==== END OF HEADER cdf.h ==== */

#endif	/* CDF_HEADER */
