#ifndef GOLSAV_HEADER
#define GOLSAV_HEADER

/* ==== HEADER golsav.h ==== */

/**
 * \file golsav.h
 * Golay-Savitzky data and first derivative smoothing.
 */

/* ANSI C, 16-Jul-1998, Andras Aszodi */

/* ---- PROTOTYPES ---- */

/**
 * gol_sav(Data, Len, Formula, Cyc, Sm, Smder): smooths the Len-long
 * array Data Cyc times using the Golay-Savitzky formula. Formula
 * is 5,7,9 or 11 (the number of G-S points): Formula<5 is rounded to 5,
 * Formula>11 down to 11, the numbers 6, 8, 10 will be rounded up.
 * If Sm!=NULL, then the smoothed points will be put into it.
 * If Smder!=NULL, then the smoothed derivatives will be put into it.
 * It is an error if both are NULL at the same time.
 * Return value: the number of smoothing cycles performed.
 */
int gol_sav(const double *Data, int Len, int Formula, int Cyc, 
    	    double *Sm, double *Smder);
	    
/* ==== END OF HEADER golsav.h ==== */

#endif	    /* GOLSAV_HEADER */
