/* ==== HEADER histo.h ==== */

#ifndef HISTO_H
#define HISTO_H

/**
 * \file histo.h
 * A simple averaging, SD and histogram-constructing function collection
 * for double data vectors.
 */

/* 1. Feb. 1993. Andras Aszodi */

/* ---- STANDARD HEADERS ---- */

#include <math.h>

/* ---- PROTOTYPES ---- */

#ifdef __cplusplus
namespace RazorBack { extern "C" {
#endif

/**
 * avg_sd(Data, Datno, Avg, Sd):
 * returns the average and SD (uncorrected, N-weighted) of a
 * data vector Data[] that is Datno long. Avg, Sd are not modified if
 * Datno<=0. Sd==1.0 if Datno=1.
 */
void avg_sd(double Data[], int Datno, double *Avg, double *Sd);

/**
 * make_hist(Data, Datno, Binno, Low, Step, Bins):
 * makes a histogram from an array Data[]. The histogram
 * is in Bins[] (number of occurrences), there are always Binno bins
 * which span the range of Data[]. The lowest bin is at Low, the 
 * width of the bins is Step.
 */
void make_hist(double Data[], int Datno, int Binno,
	double *Low, double *Step, int Bins[]);

#ifdef __cplusplus
} }
#endif

/* ==== END OF HEADER histo.h ==== */

#endif	/* HISTO_H */
