#ifndef KUIPERKOLMS_HEADER
#define KUIPERKOLMS_HEADER

/* ==== HEADER kuiperkolms.h ==== */

/**
 * \file
 * Kolmogorov-Smirnov statistics with Kuiper's modification.
 * \author Andras Aszodi
 * \date 2-Jul-2002.
 */

/* ==== TYPEDEFS ==== */

#ifdef __cplusplus
namespace RazorBack {
#endif

/**
 * The signature of a function that calculates the CDF of a distribution.
 * The first argument is the argument of the CDF, the second argument
 * can point to a data structure that contains parameters of the CDF.
 * If it is NULL, then it shall be ignored.
 */
typedef double(*Cdftype_)(double,const void*);

/* ==== PROTOTYPES ==== */

#ifdef __cplusplus
extern "C" {
#endif

/**
 * Calculates the Kuiper-modified Kolmogorov-Smirnov statistics probability
 * by comparing the SORTED discrete abscissa values in Data (n long)
 * to the known distribution given by the CDF Cdf. The parameters of Cdf
 * are optionally supplied in Cdfpars, use NULL if there are none.
 */
double kolms_vstat1(const double *Data, int n, Cdftype_ Cdf, const void* Cdfpars);

/**
 * Calculates the Kuiper-modified Kolmogorov-Smirnov statistics probability
 * by comparing the SORTED discrete abscissa values in Data1 (n1 long)
 * to the SORTED discrete abscissa values in Data2 (n2 long).
 */
double kolms_vstat2(const double *Data1, int n1, const double *Data2, int n2);

/** Sorts the n-long double array d into ascending order. */
void ascend_dblsort(double *d, int n);

#ifdef __cplusplus
} }
#endif

/* ==== END OF HEADER kuiperkolms.h ==== */

#endif	/* KUIPERKOLMS_HEADER */
