#ifndef STUDENT_HEADER
#define STUDENT_HEADER

/* ==== HEADER student.h ==== */

/**
 * \file
 * t-test and F-test for comparing two means.
 * \author Andras Aszodi
 * \date 22-Nov-2001.
 */

/* ==== STANDARD HEADERS ==== */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

/* ==== PROTOTYPES ==== */

#ifdef __cplusplus
namespace RazorBack { extern "C" {
#endif

/**
 * t_two(t, Df): returns the two-sided probability
 * that t is equal to 0 with Df degrees of freedom.
 * Returns -1.0 on error.
 */
double t_two(double t, int Df);

/**
 * t_samevar(Ave1, Ave2, Var1, Var2, n1, n2, T):
 * Student's t-test for two distributions that have the same
 * "true" variance. Ave1, 2 are the averages, Var1, 2 are the variances.
 * (These should pass an F-test for the Var1==Var2 hypothesis.)
 * n1, n2 are the number of data points. 
 * Return values: returns the probability (significance) level 
 * and also returns the t-stat value in *T if T!=NULL. Returns -1.0 if
 * something was fishy plus prints a warning on stderr.
 */
double t_samevar(double Ave1, double Ave2, double Var1, double Var2, 
	int n1, int n2, double *T);

/**
 * t_diffvar(Ave1, Ave2, Var1, Var2, n1, n2, T):
 * Student's t-test for two distributions with different
 * variances. Ave1, 2 are the averages, Var1, 2 are the variances.
 * (These should pass an F-test for the Var1!=Var2 hypothesis.)
 * n1, n2 are the number of data points. 
 * Return values: returns the probability (significance) level 
 * and also returns the t-stat value in *T if T!=NULL. Returns -1.0 if
 * something was fishy plus prints a warning on stderr.
 */
double t_diffvar(double Ave1, double Ave2, double Var1, double Var2, 
	int n1, int n2, double *T);

/**
 * f_test(Var1, Var2, n1, n2, Fval):
 * Fisher's F-test to decide whether Var1, 2 are different
 * variances. n1, n2 are the no. of data points.
 * Return values: returns the F-statistics probability (significance) 
 * level. Also returns the F-value in *Fval if Fval!=NULL.
 * Returns -1.0 if something silly happens.
 */
double f_test(double Var1, double Var2, int n1, int n2, double *Fval);

#ifdef __cplusplus
} }
#endif

/* ==== END OF HEADER student.h ==== */

#endif	/* STUDENT_HEADER */
