/* ==== FUNCTIONS histo.c ==== */

/* A simple averaging, SD and histogram-constructing function collection
 for double data vectors. */

/* ANSI C, IRIX 4.0.5, 1. Feb. 1993. Andris Aszodi */

/* ---- HEADER ---- */

#include "histo.h"

/* ---- FUNCTIONS ---- */

/*
 * avg_sd(Data, Datno, Avg, Sd):
 * returns the average and SD (uncorrected, N-weighted) of a
 * data vector Data[] that is Datno long. Avg, Sd are not modified if
 * Datno<=0. Sd==1.0 if Datno=1.
 */
void avg_sd(double Data[], int Datno, double *Avg, double *Sd)
{
    double Sx, Sx2;
    int d;

    *Avg= *Sd=0.0;
    if (Datno<=0) return;

    Sx=Sx2=0.0;
    for (d=0; d<Datno; d++)
    {
	Sx+=Data[d];
	Sx2+=(Data[d]*Data[d]);
    }
    *Avg=Sx/Datno;
    *Sd=(Datno==1)? 0.0: sqrt(fabs((Sx2-Sx*Sx/Datno)/Datno));
}
/* END of avg_sd */

/*
 * make_hist(Data, Datno, Binno, Low, Step, Bins):
 * makes a histogram from an array Data[]. The histogram
 * is in Bins[] (number of occurrences), there are always Binno bins
 * which span the range of Data[]. The lowest bin is at Low, the 
 * width of the bins is Step.
 */
void make_hist(double Data[], int Datno, int Binno,
	double *Low, double *Step, int Bins[])
{
    double Datmax;
    int d,b;

    /* init */
    *Low=HUGE_VAL; Datmax=-HUGE_VAL;
    for (b=0; b<Binno; b++) Bins[b]=0;

    /* find extremes of the data */
    for (d=0; d<Datno; d++)
    {
	if (Data[d]< *Low) *Low=Data[d];
	if (Data[d]>Datmax) Datmax=Data[d];
    }
    *Step=(Datmax- *Low)/Binno;

    /* find the correct bin for each Data item */
    for (d=0; d<Datno; d++)
    {
	b=(int)((Data[d]- *Low)/ *Step);
	if (b>=Binno) b=Binno-1;
	Bins[b]++;
    }
}
/* END of make_hist */

/* ==== END OF FUNCTIONS histo.c ==== */
