// ==== Utilsexc.hh ====

// Base class for the exceptions thrown by my various utilities.
// Implements a consistent error reporting scheme.

// 12-Dec-2000. Andras Aszodi

// ---- MODULE HEADER ----

#include"Utilsexc.hh"
using namespace RazorBack;

// ==== METHODS ====

// ---- Utilsexc_ ----

/**
 * Init with throwing function name Thnm, and fatality indicator F.
 * By default it is false, indicating that the exceptional condition
 * was not that bad.
 * Msg, if not NULL (the default) or "" may contain a verbal description
 * of the error. Use this facility when it is not worthwhile to derive
 * an exception class for the error you wish to signal.
 */
Utilsexc_::Utilsexc_(const char *Thnm, bool F, const char *Msg)
	: Throwname(Thnm), Fatal(F)
{
    ostringstream Ost;
    print_prefix(Ost);
    if (Msg!=NULL && Msg[0]!='\0')
	Ost<<Msg;
    else
	Ost<<"General Razorback exception";
    Message=Ost.str();
}

// Prints a general exception prefix string to Ost.
// Depending on the Fatal setting, the prefix is either:-
// "! FATAL RAZORBACK ERROR in: <bad_func_name>: "
// or
// "? RAZORBACK WARNING: <bad_func_name>: "
// Hidden
void Utilsexc_::print_prefix(ostringstream& Ost) const
{
    Ost<<(Fatal? "! FATAL RAZORBACK ERROR in: ": "? RAZORBACK WARNING: ")
	    <<throw_name()<<": ";
}
// END of print_prefix()

// ---- Emptyexc_ ----

Emptyexc_::Emptyexc_(const char *Thnm, bool F)
	: Utilsexc_(Thnm, F)
{
    ostringstream Ost;
    print_prefix(Ost);
    Ost<<"Empty object";
    Message=Ost.str();
}

// ==== END OF METHODS Utilsexc.cc ====
