#ifndef DERIVHDL_TMPL_METHODS
#define DERIVHDL_TMPL_METHODS

// ==== TEMPLATE METHODS Derivhdl.cc ====

// Handler and reference counting class templates
// for base-derived class hierarchies.

// 15-Jan-2002, Andras Aszodi

// ==== METHODS ====
namespace RazorBack {

// assignment
template <class BaseT_>
Derivhdl_<BaseT_>& Derivhdl_<BaseT_>::operator=(const Derivhdl_& Rhs)
{
    if (this==&Rhs || Dataptr==Rhs.Dataptr)	// same handler or same data
	return *this;  // X=X
    unlink_old();
    Dataptr=Rhs.Dataptr;  	    // shallow copy
    Dataptr->Refcount++;
    return *this;
}
// END of =

// ref(): access to the underlying BaseT_ object.
// Throws Emptyexc_ if the calling object is empty.
template <class BaseT_>
const BaseT_& Derivhdl_<BaseT_>::ref() const
{
    if (empty())
	throw Emptyexc_("Derivhdl_::ref() const");
    return *(Dataptr->Ptr);
}

template <class BaseT_>
BaseT_& Derivhdl_<BaseT_>::ref()
{
    if (empty())
	throw Emptyexc_("Derivhdl_::ref()");
    if (Dataptr->Refcount>1)	// more than one handlers
    {
	// non-const access, must be cloned
	unlink_old();
	Dataptr=new Refcount_(*Dataptr);    // deep copy
	Dataptr->Refcount=1;
    }
    return *(Dataptr->Ptr);
}
// END of ref()

// ptr(): returns a pointer to the underlying BaseT_ object.
// Returns NULL if the calling object is empty.
template <class BaseT_>
const BaseT_* Derivhdl_<BaseT_>::ptr() const
{
    if (empty()) return NULL;
    else return Dataptr->Ptr;
}

template <class BaseT_>
BaseT_* Derivhdl_<BaseT_>::ptr()
{
    if (empty()) return NULL;
    if (Dataptr->Refcount>1)	// more than one handlers
    {
	// non-const access, must be cloned
	unlink_old();
	Dataptr=new Refcount_(*Dataptr);    // deep copy
	Dataptr->Refcount=1;
    }
    return Dataptr->Ptr;
}
// END of ptr()

// unlink_old():
// unlinks old data, ie. reduces Refcount and actually deletes
// the Refcount_ object together with its BaseT_ object
// if there are no more references to it.
// Hidden
template <class BaseT_>
void Derivhdl_<BaseT_>::unlink_old()
{
    if (!empty())
    {
	Dataptr->Refcount--;
	if (!Dataptr->Refcount) // this was the last
	    delete Dataptr;    	// to the hell with it!
    }
}
// END of unlink_old()

} // RazorBack

// ==== END OF TEMPLATE METHODS Derivhdl.cc ====

#endif      // DERIVHDL_TMPL_METHODS
