#ifndef SPARSEVEC_TMPL_DEFS
#define SPARSEVEC_TMPL_DEFS

// ==== TEMPLATE DEFINITIONS Sparsevec.cc ====

// Sparse vector class template.

// 2-Nov-1998. Andras Aszodi

// ---- COMPATIBILITY NOTE ----

// This class depends on the Standard Template Library.

// ---- STANDARD HEADERS ----

#ifdef _STANDARD_C_PLUS_PLUS
#include <algorithm>
using namespace std;
#else
#include <algo.h>
#endif

// ==== Sparsevec_ METHODS ====
namespace RazorBack {

// ---- Static init ----

template <class T_>
const T_ Sparsevec_<T_>::NULLITEM=T_(0);

// get_item(Idx): returns a const ref to the Idx-th item.
// set_item(Idx, Val): stores Val at the Idx-th position.
// Does nothing if Val==0.
// Return value: true if Idx was present, false if created anew.
template <class T_>
const T_& Sparsevec_<T_>::get_item(size_t Idx) const
{
    // locate the item
    map<size_t, T_>::const_iterator Dptr=Data.find(Idx);
    if (Dptr!=Data.end())   // found
	return (*Dptr).second;     // ref to data portion
    else
	return NULLITEM;  // a "0" data
}
// END of get_item(Idx)

template <class T_>
bool Sparsevec_<T_>::set_item(size_t Idx, const T_& Val)
{
    // locate the item
    map<size_t, T_>::iterator Dptr=Data.find(Idx);
    bool Found;
    if (Found=(Dptr!=Data.end()))   // found
    {
	if (Val==NULLITEM)
	    Data.erase(Dptr);	// do not store "0"
	else
	    (*Dptr).second=Val;     // store data
    }
    else
    {
	if (Val!=NULLITEM)
	    Data[Idx]=Val;	// automagically create new storage
    }
    return Found;
}
// END of set_item(Idx, Val)

} // RazorBack

// ==== END OF TEMPLATE METHODS Sparsevec.cc ====

#endif	// SPARSEVEC_TMPL_DEFS
