#ifndef SPARSEVEC_TMPL_HEADER
#define SPARSEVEC_TMPL_HEADER

// ==== TEMPLATE HEADER Sparsevec.hh ====

/**
 * \file Sparsevec.hh
 * Sparse vector class template.
 * \author Andras Aszodi
 * \date 3-Nov-1998.
 */

// ---- COMPATIBILITY NOTE ----

// This class depends on the Standard Template Library.

// ---- STANDARD HEADERS ----

#ifdef _STANDARD_C_PLUS_PLUS
#include <map>
using namespace std;
#else
#include <map.h>
#endif

// ==== CLASSES ====

namespace RazorBack {

/**
 * Sparsevec_: template for a sparse vector class.
 * Only item access and emptiness test is provided.
 * The data type T_ must support the comparison operators <, ==
 * and must have a "null item" T_(0). Only non-null items are stored.
 */
template <class T_>
class Sparsevec_
{
    // data
    private:
	    
    static const T_ NULLITEM;	//< to be returned by const accesses
    map<size_t, T_> Data;   //< the non-0 elements keyed by their index
    
    // methods
    public:
    
    /// Ctor: inits to empty.
    Sparsevec_(): Data() {}
    
    /// get_item(Idx): returns a const ref to the Idx-th item.
    const T_& get_item(size_t Idx) const;
    
    /**
    * set_item(Idx, Val): stores Val at the Idx-th position.
    * Return value: true if Idx was present, false if created anew.
    */
    bool set_item(size_t Idx, const T_& Val);
    
    /// Resets the calling object.
    void clear() { Data.erase(Data.begin(), Data.end()); }  // clear() not in SGI STL

    /// Returns true after a clear() operation :-)
    bool empty() const { return Data.empty(); }
    
    protected:
	    
    const map<size_t, T_>& data() const { return Data; }
    
};
// END OF CLASS Sparsevec_

} // RazorBack

// ==== END OF TEMPLATE HEADER Sparsevec.hh ====

#ifdef INCLUDE_TMPL_DEFS
#include "Sparsevec.cc"
#endif

#endif	// SPARSEVEC_TMPL_HEADER
