// ==== METHODS Statexc.cc ====

// Exceptions for the statistics sub-library.

// 8-Oct-2001. Andras Aszodi

// ---- MODULE HEADERS ----

#include "Statexc.hh"
using namespace RazorBack;

// ==== METHODS ====

// ---- Toofewexc_ ----

/**
 * ctor: Thnm is the name of the method throwing the exception,
 * Name is the name of the variable that should have been more,
 * No is the number that was too few, F is the severity indicator
 * (false, the default, is non-fatal, true is fatal)
 */
Toofewexc_::Toofewexc_(const char *Thnm, const char *Name, 
    	    unsigned int No, bool F)
	: Utilsexc_(Thnm, F), Varname(Name), N(No)
{
    ostringstream Ost;
    print_prefix(Ost);
    Ost<<"Not enough data: "<<Varname<<"="<<no();
    Message=Ost.str();
}

// ---- Sdevexc_ ----

/**
 * ctor: Thnm is the name of the method throwing the exception,
 * Sdev is the bad standard deviation, F is the severity indicator
 * (false, the default, is non-fatal, true is fatal)
 */
Sdevexc_::Sdevexc_(const char *Thnm, double Sdev, bool F)
	: Utilsexc_(Thnm, F), Sd(Sdev)
{
    ostringstream Ost;
    print_prefix(Ost);
    Ost<<"Invalid standard deviation ("<<Sd<<")";
    Message=Ost.str();
}

// ---- Forgottenexc_ ----

/**
 * ctor: Thnm is the name of the method throwing the exception,
 * Forgotnm is the name of the method that we forgot to invoke,
 * F is the severity indicator
 * (false, the default, is non-fatal, true is fatal)
 */
Forgottenexc_::Forgottenexc_(const char *Thnm, const char *Forgotnm, bool F)
	: Utilsexc_(Thnm, F), Forgotten(Forgotnm)
{
    ostringstream Ost;
    print_prefix(Ost);
    Ost<<"Must invoke \""<<forgotten_name()<<"\" first!";
    Message=Ost.str();
}

// ==== END OF METHODS Statexc.cc ====
