#ifndef LINREG_HEADER
#define LINREG_HEADER

// ==== HEADER Linreg.hh ====

/**
 * \file Linreg.hh
 * Linear regression.
 * The implementation is based on the old and sometimes unstable
 * "normal equations" method.
 */

// 19-Sep-2001, Andras Aszodi

// ---- MODULE HEADERS ----

#include"Paramest.hh"

// ==== CLASSES ====

namespace RazorBack {

/// Class Linreg_: class for linear regression.
class Linreg_: public Paramest_
{
    // data
    private:
	    
    Vecdbl_ W;	///< weight vector
    
    // methods
    public:
	    
    /// Init to empty.
    Linreg_(): Paramest_() {}
    
    /**
    * Init to desired sizes. 
    * Actual regression calls may change the settings
    */
    Linreg_(size_t NM, size_t NP): Paramest_(NM, NP), W(NM) {}
    
    /**
    * lin_reg(Xmeas, Ymeas [,Userwgt]): multivariate linear regression
    * calculating the parameters P in the equation
    * Y=P[0]*X[0]+...+P[Np-1]*X[Np-1].
    * Xmeas is an Nm x Np matrix holding Nm rows of Np independent vars,
    * Ymeas is an Nm-long vector holding the dependent vars.
    * Userwgt is an optional Nm-long vector of weights.
    * Throws Dimexc_ and Toofewexc_.
    * Return value: the residual deviation.
    * Invoke param(), sdev(), correl() for the detailed results.
    */
    double lin_reg(const Rmatdbl_& Xmeas, const Vecdbl_& Ymeas, const Vecdbl_& Userwgt);
    double lin_reg(const Rmatdbl_& Xmeas, const Vecdbl_& Ymeas);
    
    // hidden methods
    protected:
	    
    void adjust_dimensions(const Rmatdbl_& Xmeas, const Vecdbl_& Ymeas, const Vecdbl_& Userwgt);
    void setup_weight(const Vecdbl_& YW);
    void setup_xtx(const Rmatdbl_& Xmeas, const Vecdbl_& Ymeas);
    double quality(const Rmatdbl_& Xmeas, const Vecdbl_& Ymeas);
    
};
// END OF CLASS Linreg_

} // RazorBack

// ==== END OF HEADER Linreg.hh ====

#endif	    // LINREG_HEADER
