#ifndef STATEXC_CLASS_HEADER
#define STATEXC_CLASS_HEADER

// ==== HEADER Statexc.hh ====

/**
 * \file Statexc.hh
 * Exceptions for the statistics sub-library.
 */

// 8-Oct-2001. Andras Aszodi

// ---- STANDARD HEADERS ----

#include <stdlib.h>

#ifdef _STANDARD_C_PLUS_PLUS
#include <iostream>
using namespace std;
#else
#include <iostream.h>
#endif

// ---- MODULE HEADERS ----

#include "Utilsexc.hh"

// ==== CLASSES ====

namespace RazorBack {

/**
 * Class Toofewexc_: an exception object
 * that stores an unsigned integer and can be thrown
 * if there was not enough data available.
 * Usage: throw(Statexc_("bad_function", N)
 */
class Toofewexc_: public Utilsexc_
{
    // data
    private:
    
    string Varname; 	///< the thing that was not enough
    unsigned int N;  ///< the number that was too low
    
    // methods
    public:
    
    /**
     * ctor: Thnm is the name of the method throwing the exception,
     * Name is the name of the variable that should have been more,
     * No is the number that was too few, F is the severity indicator
     * (false, the default, is non-fatal, true is fatal)
     */
    explicit Toofewexc_(const char *Thnm, const char *Name, 
    	    unsigned int No=0, bool F=false);
    
    /// Returns the name of the thing that should have been higher.
    const string& varname() const { return Varname; }
    
    /// Returns the number that was not high enough.
    unsigned int no() const { return N; }
    
    private:
    Toofewexc_();   // no default ctor
};
// END of exception class Toofewexc_

/**
 * Class Sdevexc_: an exception object
 * that stores an invalid standard deviation value (usually <=0.0).
 * Usage: throw(Statexc_("bad_function", Sdev)
 */
class Sdevexc_: public Utilsexc_
{
    // data
    private:
	    
    double Sd;	    ///< the bad standard deviation value
    
    // methods
    public:
	    
    /**
     * ctor: Thnm is the name of the method throwing the exception,
     * Sdev is the bad standard deviation, F is the severity indicator
     * (false, the default, is non-fatal, true is fatal)
     */
    explicit Sdevexc_(const char *Thnm, double Sdev=0.0, bool F=false);
    
    /// Returns the bad standard deviation value.
    double sd() const { return Sd; }
    
    private:
    Sdevexc_();     // no default ctor
};
// END of exception class Sdevexc_

class Forgottenexc_: public Utilsexc_
{
    // data
    private:
	    
    string Forgotten;	///< name of the method we forgot to invoke
    
    // methods
    public:
	    
    /**
     * ctor: Thnm is the name of the method throwing the exception,
     * Forgotnm is the name of the method that we forgot to invoke,
     * F is the severity indicator
     * (false, the default, is non-fatal, true is fatal)
     */
    explicit Forgottenexc_(const char *Thnm, const char *Forgotnm, bool F=false);
    
    /// Returns the name of the forgotten method.
    const string& forgotten_name() const { return Forgotten; }
    
    private:
    Forgottenexc_();	// no default ctor
};
// END of exception class Forgottenexc_

} // RazorBack

// ==== END OF CLASS HEADER Statexc.hh ====

#endif      // STATEXC_CLASS_HEADER
