#ifndef CLU_HEADER
#define CLU_HEADER

/* ==== HEADER clu.h ==== */

/**
 * \file clu.h
 * Simple hierarchical clustering.
 */

/* 10-Nov-2000. Andras Aszodi */

/* ---- UTILITY HEADERS ---- */

#include "mattri.h"

/* ---- TYPEDEFS ---- */

#ifdef __cplusplus
namespace RazorBack {
#endif

/**
 * Clu_ : a cluster structure. The things to be clustered
 * are represented by integers running from 0 upwards.
 * The clusters are organised into a binary tree; non-leaf
 * nodes always have 2 sub-nodes.
 */
typedef struct clu_
{
    int *Members;   /**< the indices of the members of the cluster */
    int No;	/**< no. of members */
    struct clu_ *Sub1, *Sub2;	/**< ptrs to subclusters: NULL for leaves */
    double Lid, Sed;	/**< largest internal and smallest external distance */
} Clu_;

/* ---- PROTOTYPES ---- */

#ifdef __cplusplus
extern "C" {
#endif

/**
 * make_clus(Dist, Cluno):
 * performs the clustering. The distances between the
 * things to be clustered are in the Dist matrix: the [i][j]:th
 * entry corresponds to the (non-negative) distance between the i and j.
 * Only the lower triangle is used, i>=j. Self-distances are 0.0.
 * Return value: a forest of Clus_ struct trees describing the clusters
 * (allocated within). The no. of cluster trees is returned in *Cluno.
 * Returns NULL on error.
 */
Clu_ *make_clus(const Mattri_ *Dist, int *Cluno);

/**
 * free_clusters(Clus, Cluno): 
 * frees up the cluster "forest" (i.e. the
 * array of cluster trees generated by make_clus(...)).
 */
void free_clusters(Clu_ Clus[], int Cluno);

/**
 * print_clus(Clu, Out): 
 * prints the whole cluster tree with root pointed
 * to by Clu to Out.
 */
void print_clus(const Clu_ *Clu, FILE *Out);

#ifdef __cplusplus
} }
#endif

/* ==== END OF HEADER clu.h ==== */

#endif	/* CLU_HEADER */
