#ifndef RANDOM_HEADER
#define RANDOM_HEADER

/* ==== MODULE HEADER random.h ==== */

/**
 * \file
 * Uniformly distributed random numbers using the 32-bit generator of
 * L'Ecuyer, P. Efficient and portable combined random number
 * generators, C.A.C.M., vol. 31, 742-749, June 1988.
 * Period is very long, about 2.3e18.
 * Plus the Box/Mueller method for standard normal variates.
 * \author Andras Aszodi
 * \date 20-Nov-2001.
 */

/* ==== PROTOTYPES ==== */

#ifdef __cplusplus
namespace RazorBack { extern "C" {
#endif

/**
 * Initialises the uniform random number generator with two seeds.
 * If S1==0, then the value returned by getpid() is used instead.
 * If S2==0, then the value returned by time(NULL) is used instead.
 */
void uniform_init(int S1, int S2);

/** Returns a uniformly distributed integer in the range [0 .. 2147483563]. */
int int_uniform(void);

/** Returns a uniformly distributed real value between [0.0 .. 1.0]. */
double dbl_uniform(void);

/** Returns a random number distributed as Standard Normal. */
double random_gauss(void);

#ifdef __cplusplus
} }
#endif
/* ==== END OF MODULE HEADER random.h ==== */

#endif	    /* RANDOM_HEADER */
